// 230328

module StrobeGenerator #(
	parameter	FREQUENCY_HZ	= 10_000_000,
	parameter	PERIOD_US		= 100
)(
	input		Clock,
	input		Reset,
	output reg	Strobe
);
	
	localparam DELAY = (FREQUENCY_HZ / 1_000_000) * PERIOD_US - 1;
	localparam WIDTH = $clog2(DELAY + 1);
	
	initial begin
		if(DELAY <= 0)
			$fatal(0, "Wrong DELAY value: %d", DELAY);
	end
	
	reg [WIDTH-1:0] Counter;
	
	always @(posedge Clock, negedge Reset) begin
		if(!Reset) begin
			Counter	<= DELAY;
			Strobe	<= 1'b0;
		end else if(!Counter) begin
			Counter	<= DELAY;
			Strobe	<= 1'b1;
		end else begin
			Counter	<= Counter - 1'b1;
			Strobe	<= 1'b0;
		end 
	end
	
endmodule
