// 230328

`timescale 1ns/1ps	// time-unit, precision

module StrobeGenerator_tb();
	
	parameter CLOCK_FREQ_HZ	= 10_000_000;
	parameter HALF_PERIOD_NS = 1_000_000_000 / (2 * CLOCK_FREQ_HZ);
	
	// Clock generator
	reg Clock = 1'b1;
	always begin
		#HALF_PERIOD_NS;
		Clock = !Clock;
	end
	
	reg	Reset = 1'b0;
	wire Strobe;

	initial begin
		$timeformat(-6, 3, "us", 10);
		$display("===== START =====");
		$display("FREQUENCY_HZ = %9d", DUT.FREQUENCY_HZ);
		$display("PERIOD_US    = %9d", DUT.PERIOD_US);
		$display("DELAY        = %9d", DUT.DELAY);
		$display("WIDTH        = %9d", DUT.WIDTH);
		
		#1 Reset = 1'b1;
		
		repeat(3) begin
			@(posedge Strobe);
			$display("Strobe detected at %t", $realtime);
		end
		
		@(negedge Strobe);
		#1000;
		
		$display("===== END =====");
		#1 $finish;
	end
	
	// Instantiate device under test
	StrobeGenerator #(
		.FREQUENCY_HZ(CLOCK_FREQ_HZ),
		.PERIOD_US(10)
	) DUT(
		.Clock(Clock),
		.Reset(Reset),
		.Strobe(Strobe)
	);
	
	initial begin
		$dumpfile("strobe_generator.vcd");
		$dumpvars(0, DUT);
	end

endmodule
