module top(
	input			Reset,
	input	[3:0]	KeyRows,
	output	[7:0]	Cathodes,
	output	[7:0]	Segments,
	output			LedKeyPressed
);

	// Generator sygnau zegarowego
	parameter FREQUENCY_HZ = 14_000_000;
	wire Clock;
	OSCH #(
		.NOM_FREQ("14.00")
	) OSCH_inst(
		.STDBY(1'b0),
		.OSC(Clock),
		.SEDSTDBY()
	);
	
	// Rejestr Data przechowuje 4 ostatnio zarejestrowane scankody
	reg [19:0] Data;
	wire [4:0] KeyCode;
	wire KeyStrobe;
	always @(posedge Clock, negedge Reset)
		if(!Reset)
			Data <= 0;
		else if(KeyStrobe)
		Data <= {Data[14:0], KeyCode};
			
	// Przecinki oddzielajce poszczeglne scankody
	wire [7:0] DecimalPoints = 8'b01010101;
	
	// Glue logic
	wire SwitchCathode;
	wire [31:0] DataToDisplay = {
		3'b000, Data[19:15],
		3'b000, Data[14:10],
		3'b000, Data[ 9: 5],
		3'b000, Data[ 4: 0]
	};
	
	// Instancja sterownika wywietlacza multipleksowanego
	DisplayMultiplex #(
		.FREQUENCY_HZ(FREQUENCY_HZ),
		.SWITCH_PERIOD_US(2000),
		.DIGITS(8)
	) DisplayMultiplex0(
		.Clock(Clock),
		.Reset(Reset),
		.Data(DataToDisplay),
		.DecimalPoints(DecimalPoints),
		.Cathodes(Cathodes),
		.Segments(Segments),
		.SwitchCathode(SwitchCathode)
	);
	
	// Instancja sterownika klawiatury matrycowej
	MatrixKeyboard MatrixKeyboard0(
		.Clock(Clock),
		.Reset(Reset),
		.Cathodes(Cathodes),
		.Rows(KeyRows),
		.SwitchCathode(SwitchCathode),
		.KeyCode(KeyCode),
		.KeyStrobe(KeyStrobe),
		.KeyPressed(LedKeyPressed)
	);
	

endmodule
